package com.ruoyi.web.controller.dragForm;

import com.ruoyi.business.entity.DragTableStyle;
import com.ruoyi.business.entity.vo.DragTableVo;
import com.ruoyi.business.service.IDragTableStyleService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 动态格样式Controller
 * 
 * @author ruoyi
 * @date 2023-11-03
 */
@RestController
@RequestMapping("/system/style")
public class DragTableStyleController extends BaseController
{
    @Autowired
    private IDragTableStyleService dragTableStyleService;

    /**
     * 查询动态格样式列表
     */
    @PreAuthorize("@ss.hasPermi('system:style:list')")
    @GetMapping("/list")
    public TableDataInfo list(DragTableStyle dragTableStyle)
    {
        startPage();
        List<DragTableStyle> list = dragTableStyleService.selectDragTableStyleList(dragTableStyle);
        return getDataTable(list);
    }

    /**
     * 导出动态格样式列表
     */
    @PreAuthorize("@ss.hasPermi('system:style:export')")
    @Log(title = "动态格样式", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, DragTableStyle dragTableStyle)
    {
        List<DragTableStyle> list = dragTableStyleService.selectDragTableStyleList(dragTableStyle);
        ExcelUtil<DragTableStyle> util = new ExcelUtil<DragTableStyle>(DragTableStyle.class);
        util.exportExcel(response, list, "动态格样式数据");
    }

    /**
     * 获取动态格样式详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:style:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(dragTableStyleService.selectDragTableStyleById(id));
    }

    /**
     * 新增动态格样式
     */
    @PreAuthorize("@ss.hasPermi('system:style:add')")
    @Log(title = "动态格样式", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody DragTableVo vo)
    {
        return toAjax(dragTableStyleService.batchInsertDragTableStyle(vo));
    }

    /**
     * 修改动态格样式
     */
    @PreAuthorize("@ss.hasPermi('system:style:edit')")
    @Log(title = "动态格样式", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody DragTableVo vo)
    {
        dragTableStyleService.updateDragTableStyle(vo);
        return AjaxResult.success();
    }

    /**
     * 删除动态格样式
     */
    @PreAuthorize("@ss.hasPermi('system:style:remove')")
    @Log(title = "动态格样式", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(dragTableStyleService.deleteDragTableStyleByIds(ids));
    }

}
