package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.SysBpmNodeScript;
import com.ruoyi.system.service.ISysBpmNodeScriptService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 流程节点脚本Controller
 *
 * @author ruoyi
 * @date 2023-10-26
 */
@RestController
@RequestMapping("/system/script")
@Api(value = "/system/script")
public class SysBpmNodeScriptController extends BaseController
{
    @Autowired
    private ISysBpmNodeScriptService sysBpmNodeScriptService;

    /**
     * 查询流程节点脚本列表
     */
//    @PreAuthorize("@ss.hasPermi('system:script:list')")
    @GetMapping("/list")
    @ApiOperation(value = "查询流程节点脚本列表")
    public TableDataInfo list(SysBpmNodeScript sysBpmNodeScript)
    {
        startPage();
        List<SysBpmNodeScript> list = sysBpmNodeScriptService.selectSysBpmNodeScriptList(sysBpmNodeScript);
        return getDataTable(list);
    }

    /**
     * 导出流程节点脚本列表
     */
//    @PreAuthorize("@ss.hasPermi('system:script:export')")
    @Log(title = "流程节点脚本", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    @ApiOperation(value = "导出流程节点脚本列表")
    public void export(HttpServletResponse response, SysBpmNodeScript sysBpmNodeScript)
    {
        List<SysBpmNodeScript> list = sysBpmNodeScriptService.selectSysBpmNodeScriptList(sysBpmNodeScript);
        ExcelUtil<SysBpmNodeScript> util = new ExcelUtil<SysBpmNodeScript>(SysBpmNodeScript.class);
        util.exportExcel(response, list, "流程节点脚本数据");
    }

    /**
     * 获取流程节点脚本详细信息
     */
//    @PreAuthorize("@ss.hasPermi('system:script:query')")
    @GetMapping(value = "/{id}")
    @ApiOperation(value = "获取流程节点脚本详细信息")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(sysBpmNodeScriptService.selectSysBpmNodeScriptById(id));
    }

    /**
     * 新增流程节点脚本
     */
//    @PreAuthorize("@ss.hasPermi('system:script:add')")
    @Log(title = "流程节点脚本", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation(value = "新增流程节点脚本")
    public AjaxResult add(@RequestBody SysBpmNodeScript sysBpmNodeScript)
    {
        return toAjax(sysBpmNodeScriptService.insertSysBpmNodeScript(sysBpmNodeScript));
    }

    /**
     * 修改流程节点脚本
     */
//    @PreAuthorize("@ss.hasPermi('system:script:edit')")
    @Log(title = "流程节点脚本", businessType = BusinessType.UPDATE)
    @PutMapping
    @ApiOperation(value = "修改流程节点脚本")
    public AjaxResult edit(@RequestBody SysBpmNodeScript sysBpmNodeScript)
    {
        return toAjax(sysBpmNodeScriptService.updateSysBpmNodeScript(sysBpmNodeScript));
    }

    /**
     * 删除流程节点脚本
     */
//    @PreAuthorize("@ss.hasPermi('system:script:remove')")
    @Log(title = "流程节点脚本", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    @ApiOperation(value = "删除流程节点脚本")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(sysBpmNodeScriptService.deleteSysBpmNodeScriptByIds(ids));
    }
}
