package com.ruoyi.web.controller.system;

import java.util.List;
import java.util.Set;

import com.ruoyi.common.core.domain.entity.DataSource;
import com.ruoyi.common.core.domain.entity.SysTenant;
import com.ruoyi.system.service.IDataSourceService;
import com.ruoyi.system.service.impl.SysLogininforServiceImpl;
import com.ruoyi.system.service.impl.SysTenantServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysMenu;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginBody;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.framework.web.service.SysLoginService;
import com.ruoyi.framework.web.service.SysPermissionService;
import com.ruoyi.system.service.ISysMenuService;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;

/**
 * 登录验证
 *
 * @author ruoyi
 */
@RestController
public class SysLoginController {


    @Autowired
    private SysLoginService loginService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private SysPermissionService permissionService;

    @Resource
    private SysTenantServiceImpl sysTenantService;

    @Resource
    private IDataSourceService dataSourceService;

    /**
     * 数据引擎切换数据源接口地址
     */
    @Value("${parameter.ip.DATA_ENGINE_IP}")
    public String DATA_ENGINE_IP;

    /**
     * 动态表单切换数据源接口地址
     */
    @Value("${parameter.ip.DRAG_FORM_IP}")
    public String DRAG_FORM_IP;

    /**
     * 流程引擎切换数据源地址接口
     */
    @Value("${parameter.ip.PROCESS_ENGINE_IP}")
    public String PROCESS_ENGINE_IP;

    RestTemplate restTemplate = new RestTemplate();


    /**
     * 登录方法
     *
     * @param loginBody 登录信息
     * @return 结果
     */
    @PostMapping("/login")
    public AjaxResult login(@RequestBody LoginBody loginBody) {
        //校验租户状态?生成token
        AjaxResult ajax = AjaxResult.success();
        loginService.validateCaptcha(loginBody.getUsername(),loginBody.getCode(),loginBody.getUuid());
        if(loginBody.getUsername().equals("admin")){
          return AjaxResult.error("未有此用户信息");
        }
        String  checkTenantExpirationTimeMsg= loginService.checkTenantExpirationTime(loginBody.getUsername());
        if(!checkTenantExpirationTimeMsg.isEmpty()){
            return AjaxResult.error(checkTenantExpirationTimeMsg);
        }
        // 生成令牌
        String token = loginService.login(loginBody.getUsername(), loginBody.getPassword(), loginBody.getCode(),
                loginBody.getUuid());
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    /**
     * 获取用户信息
     *
     * @return 用户信息
     */
    @GetMapping("getInfo")
    public AjaxResult getInfo() {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        // 角色集合
        Set<String> roles = permissionService.getRolePermission(user);
        // 权限集合
        Set<String> permissions = permissionService.getMenuPermission(user);
        AjaxResult ajax = AjaxResult.success();
        ajax.put("user", user);
        ajax.put("roles", roles);
        ajax.put("permissions", permissions);
        //租户信息
        if (user.getTenantId() != null) {
            //租户信息
            SysTenant sysTenant = sysTenantService.selectSysTenantByTenantId(user.getTenantId());
            ajax.put("tenant", sysTenant);
            //数据源信息
            DataSource dataSource = dataSourceService.selectById(sysTenant.getDatasourceId());
            ajax.put("dataSource", dataSource);
        }
        return ajax;
    }

    /**
     * 获取路由信息
     *
     * @return 路由信息
     */
    @GetMapping("getRouters")
    public AjaxResult getRouters() {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuTreeByUserId(userId);
        return AjaxResult.success(menuService.buildMenus(menus));
    }
}
